"""
Machine types commands
"""
from prettytable import PrettyTable

import click


@click.group(name='machine-types', help='Machine (hardware) commands group')
@click.pass_context
def machine_types(ctx):
    pass


@machine_types.command('list', help='Show available machine types')
@click.pass_context
def list_machine_types(ctx):

    conn = ctx.obj['connection']

    url = '{root}/machine_types'.format(
        root=conn.URL
    )

    r = conn.get(url)
    if r.status_code == 200:
        resp = r.json()
    else:
        print('Error: {}'.format(r.status_code))
        return

    # Make a summary table

    if len(resp) == 0:
        print('No available machine types')
        return list()

    summary = PrettyTable(border=False)
    summary.field_names = ['ID', 'SPECS']
    summary.align = 'l'
    for machine_type in resp:
        summary.add_row([machine_type['uid'], machine_type['name']])
    print(summary)

    return [machine_type['uid'] for machine_type in resp]




"""
Job commands
"""
import os
import sys
import json
import subprocess
from prettytable import PrettyTable

import click
import websocket

from onepanel.commands.login import login_required
from onepanel.commands.projects import Project


class Job:
    """
    Data model
    """

    uid = str
    machine_type = str
    environment = str
    project = Project

    def __init__(self, uid=None, project=None):

        self.uid = uid
        self.project = project


@click.group(help='Job commands group')
@click.pass_context
def jobs(ctx):

    # Retrieve the project from the local storage and pass it over
    local_project = Project.from_directory(os.getcwd())
    if local_project is None:
        # All job commands require project_uid. Return error before reaching them:
        print('This project is not initialized, type "onepanel init" to initialize this project')
        sys.exit(1)
    else:
        ctx.obj['project'] = local_project


@jobs.command('create', help='Execute a command on a remote machine in the current project')
@click.argument(
    'command',
    type=str
)
@click.option(
    '-m', '--machine-type',
    type=str,
    required=True,
    help='Machine type ID'
)
@click.option(
    '-e', '--environment',
    type=str,
    required=True,
    help='Instance template ID'
)
@click.pass_context
@login_required
def create_job(ctx, command, machine_type, environment):

    conn = ctx.obj['connection']
    project = ctx.obj['project']

    request_data = {
        'command': command,
        'machineType': {
            'uid': machine_type
        },
        'instanceTemplate': {
            'uid': environment
        }
    }

    url = '{root}/accounts/{account_uid}/projects/{project_uid}/jobs'.format(
        root=conn.URL,
        account_uid=project.account_uid,
        project_uid=project.project_uid
    )

    r = conn.post(url, data=json.dumps(request_data))
    if r.status_code == 200:
        resp = r.json()
    else:
        print('Error: {}'.format(r.status_code))
        return

    job_uid = resp['uid']

    print('New job created: {}'.format(job_uid))

    return job_uid


@jobs.command('list', help='Show commands executed on remote machines')
@click.option(
    '-a', '--all',
    type=bool,
    is_flag=True,
    default=False,
    help='Include finished commands'
)
@click.pass_context
@login_required
def list_jobs(ctx, all):

    conn = ctx.obj['connection']
    project = ctx.obj['project']

    account_uid = project.account_uid
    project_uid = project.project_uid

    # Request

    url = '{root}/accounts/{account_uid}/projects/{project_uid}/jobs{params}'.format(
        root=conn.URL,
        account_uid=account_uid,
        project_uid=project_uid,
        params='?running=true' if not all else ''
    )

    r = conn.get(url)
    if r.status_code == 200:
        resp = r.json()
    else:
        print('Error: {}'.format(r.status_code))
        return

    # Make a summary table

    if len(resp) == 0:
        print('No jobs. Use "--all" flag to retrieve completed jobs.')
        return None

    summary = PrettyTable(border=False)
    summary.field_names = ['ID', 'COMMAND']
    summary.align = 'l'
    for job in resp:
        summary.add_row([job['uid'], job['command']])
    print(summary)

    return [job['uid'] for job in resp]


@jobs.command('stop', help='Stop a job')
@click.argument(
    'job_uid',
    type=str
)
@click.pass_context
@login_required
def kill_job(ctx, job_uid):

    conn = ctx.obj['connection']
    project = ctx.obj['project']

    url = '{root}/accounts/{account_uid}/projects/{project_uid}/jobs/{job_uid}'.format(
        root=conn.URL,
        account_uid=project.account_uid,
        project_uid=project.project_uid,
        job_uid=job_uid
    )

    r = conn.delete(url)
    c = r.status_code
    if c == 200:
        print('Job stopped: {}'.format(job_uid))
        return True
    elif c == 404:
        print('Job not found: {}'.format(job_uid))
    else:
        print('Error: {}'.format(c))

    return False


@jobs.command('logs', help='Show a log of the command')
@click.argument(
    'job_uid',
    type=str
)
@click.pass_context
@login_required
def job_logs(ctx, job_uid):

    conn = ctx.obj['connection']
    project = ctx.obj['project']

    url = '{root}/accounts/{account_uid}/projects/{project_uid}/jobs/{job_uid}/logs'.format(
        root=conn.URL,
        account_uid=project.account_uid,
        project_uid=project.project_uid,
        job_uid=job_uid
    )

    r = conn.get(url)
    c = r.status_code

    if c == 200:

        resp_text = r.text
        # Response contains escaped characters. E.g., r.content is:
        # b'"rm: cannot remove \'README.md\': No such file or directory\\n"'
        # Clean them first
        log_lines = resp_text[1:-1].split('\\n')
        print('\n'.join(log_lines), end='')
        return True

    elif c == 400:

        # Streaming via WebSocket
        # See https://pypi.python.org/pypi/websocket-client/

        streaming_url = '{ws_root}/accounts/{account_uid}/projects/{project_uid}/jobs/{job_uid}/logs?access_token={token}&tail_lines=1'.format(
            ws_root='wss://c.onepanel.io/api',
            account_uid=project.account_uid,
            project_uid=project.project_uid,
            job_uid=job_uid,
            token=conn.token
        )

        ws = websocket.WebSocketApp(
            url=streaming_url,
            header=conn.headers,
            on_message=lambda ws, message: print(message, end=''),
            on_error=print
        )
        ws.run_forever()

    elif c == 404:

        print('Job not found: {}'.format(job_uid))

    else:

        print('Error: {}'.format(c))

    return False

@jobs.command(
    'download-output',
    help='Download output for a job. RESOURCE for job output: '
         '"jobs/__JOB_ID__/output" OR "__ACCOUNT_ID__/__PROJECT_ID__/jobs/__JOB_ID__/output".'
)
@click.argument(
    'resource',
    type=click.Path(),
    required=True
)
@click.argument(
    'directory',
    type=click.Path(),
    required=False
)
@click.pass_context
@login_required
def jobs_download_output(ctx, resource, directory):
    project = ctx.obj['project']

    # These will be updated from `resource`
    account_uid = project.account_uid
    project_uid = project.project_uid
    job_uid = ''

    #
    # Resource
    #
    dirs = resource.split('/')

    # Job output: Method 1
    # <job_uid>
    if (len(dirs) == 1):
        job_uid = dirs[0]
    # Job output: Method 2
    # <account_uid>/projects/<project_uid>/jobs/<job_uid>
    elif len(dirs) == 5:
        try:
            account_uid, projects_dir, project_uid, jobs_dir, job_uid = dirs
            assert (projects_dir == 'projects') and (jobs_dir == 'jobs')
        except:
            print('Incorrect job path')
            return None
    else:
        print('Incorrect job uid')
        return None

    #
    # Directory
    #
    if directory is None or directory == '.':
        home = os.getcwd()
    else:
        home = os.path.join(os.getcwd(), directory)

    #
    # Clone
    #
    cwd = os.getcwd()

    cmd = (
            'rm -rf .onepanel_output'   # Cleaning after previous errors to avoid "is not an empty directory" error
            ' && git clone --quiet --no-checkout https://{host}/{account_uid}/{project_uid}-output.git .onepanel_output'
            ' && cd .onepanel_output'
            ' && git config --local core.sparseCheckout true'
            ' && echo \'jobs/{job_uid}/output/*\'> .git/info/sparse-checkout'
            ' && git checkout --quiet job-{job_uid}'
            ' && cp -r jobs/{job_uid}/output/ {dir}'
            ' && cd {cwd}'
            ' && rm -rf .onepanel_output'
        ).format(
            host='git.onepanel.io',
            account_uid=account_uid,
            project_uid=project_uid,
            job_uid=job_uid,
            dir=home,
            cwd=cwd
        )
    p = subprocess.Popen(cmd, shell=True)
    p.wait()

    if p.returncode == 0:
        print('The resource downloaded to: {dir}'.format(dir=home))
        return True
    else:
        print('Unable to download')
        return False


import click

from onepanel.commands.base import APIViewController


class VolumeTypeViewController(APIViewController):

    def __init__(self, conn):

        super(VolumeTypeViewController, self).__init__(conn)

        self.endpoint = '{root}/volume_types'.format(
            root=self.conn.URL
        )


@click.group(name='volume-types', help='Available volume types')
@click.pass_context
def volume_types(ctx):

    ctx.obj['view_controller'] = VolumeTypeViewController(ctx.obj['connection'])


@volume_types.command(
    'list',
    help='Show available volume types and their IDs'
)
@click.pass_context
def list_volume_types(ctx):

    vc = ctx.obj['view_controller']

    items = vc.get()
    vc.print_items(items, fields=['uid', 'name'], field_names=['ID', 'SPECS'])




"""
Machine types commands
"""
from prettytable import PrettyTable

import click


@click.group(help='Environment (machine types) commands group')
@click.pass_context
def environments(ctx):
    pass


@environments.command('list', help='Show available environments')
@click.pass_context
def list_environments(ctx):

    conn = ctx.obj['connection']

    url = '{root}/instance_templates?instance_type=mod'.format(
        root=conn.URL
    )

    r = conn.get(url)
    if r.status_code == 200:
        resp = r.json()
    else:
        print('Error: {}'.format(r.status_code))
        return

    # Make a summary table

    if len(resp) == 0:
        print('No available environments')
        return list()

    summary = PrettyTable(border=False)
    summary.field_names = ['ID', 'ENVIRONMENT']
    summary.align = 'l'
    for environment in resp:
        summary.add_row([environment['uid'], environment['name']])
    print(summary)

    return [environment['uid'] for environment in resp]




"""
View-Controller class for OnePanel API
"""
import os
import sys
import json
from prettytable import PrettyTable

from onepanel.commands.projects import Project


class APIViewController:

    conn = None     # Connection from onepanel.cli
    endpoint = str

    def __init__(self, conn):

        self.conn = conn

    # Wrappers for HTTP requests

    def get(self):
        """Get a JSON list from the endpoint"""

        r = self.conn.get(self.endpoint)

        c = r.status_code
        if c == 200:
            items = r.json()
        else:
            return None

        return items

    def post(self, post_object):
        """POST an object and return the newly created resource"""

        r = self.conn.post(self.endpoint, data=json.dumps(post_object))

        c = r.status_code
        if c == 200:
            created_object = r.json()
        else:
            print('Error: {}'.format(c))
            return None

        return created_object

    def delete(self, uid, message_on_success='Resource deleted'):

        resource = '{}/{}'.format(self.endpoint, uid)

        r = self.conn.delete(resource)

        c = r.status_code
        if c == 200:
            print('{message}: {id}'.format(message=message_on_success, id=uid))
            return True
        elif c == 404:
            print('Not found: {}'.format(uid))
        else:
            print('Error: {}'.format(c))

        return False

    # Support functions

    @staticmethod
    def get_project():
        """Retrieve the project from the local storage and return it"""

        local_project = Project.from_directory(os.getcwd())

        if local_project is None:
            # Return error before reaching other CLI commands:
            print('This project is not initialized, type "onepanel init" to initialize this project')
            sys.exit(1)
        else:
            return local_project

    @staticmethod
    def print_items(
            items,
            fields=None,
            field_names=None,
            empty_message='No items found'
    ):
        """A standard table for showing response to GET requests"""

        if fields is None:
            fields = ['uid', 'name']
        if field_names is None:
            field_names=['ID', 'NAME']

        if len(items) == 0:
            print(empty_message)
            return items

        summary = PrettyTable(border=False)
        summary.field_names = field_names
        summary.align = 'l'
        for item in items:
            summary.add_row([item[field] for field in fields])
        print(summary)




""" Command line interface for the OnePanel Machine Learning platform

Wrap git commands and provide transparent integration onepanel commands with the git.
"""

import os
import subprocess


class GitWrapper:
    def __init__(self):
        self.HOST = 'git.onepanel.io'

    def get_credential_helper(self):
        """ Retrieve current git credential manager and use it for storing usernames and passwords
        """
        git_cmd = 'git config credential.helper'
        p = subprocess.Popen(git_cmd, stdin=subprocess.PIPE, stdout=subprocess.PIPE, shell=True)
        p.wait()

        line = p.stdout.readline()
        credential_helper = line.decode().rstrip()
        return credential_helper

    def save_credentials(self, username, password):
        """ Store password in the current git credential storage
        """

        credential_helper = self.get_credential_helper()
        if not credential_helper:
            print('ERROR. Cannot store git username/password.\n')
            print('Git credential helper is not configured')
            return

        # we can call helper by absolute path or from git command
        if not os.path.isabs(credential_helper):
            git_cmd = 'git credential-{} store'.format(credential_helper)
        else:
            git_cmd = credential_helper

        p = subprocess.Popen(git_cmd, stdin=subprocess.PIPE, stdout=subprocess.PIPE, shell=True)
        p.stdin.write('protocol=https\n'.encode())
        p.stdin.write('host={}\n'.format(self.HOST).encode())
        p.stdin.write('username={}\n'.format(username).encode())
        p.stdin.write('password={}\n'.format(password).encode())
        p.stdin.close()
        p.wait()

    def init(self, home, account_uid, project_uid):
        git_cmd = 'git init'
        p = subprocess.Popen(git_cmd, cwd=home, shell=True)
        p.wait()

        git_cmd = 'git remote add onepanel https://{}/{}/{}.git'.format(self.HOST, account_uid, project_uid)
        p = subprocess.Popen(git_cmd, cwd=home, shell=True)
        p.wait()

    def clone(self, home, account_uid, project_uid, ext=''):
        git_cmd = 'git clone -o onepanel https://{}/{}/{}{}.git {}'.format(self.HOST, account_uid, project_uid, ext, home)
        p = subprocess.Popen(git_cmd, shell=True)
        p.wait()

        return p.returncode

    def exclude(self, home, files):
        exclude_file = os.path.join(home, '.git/info/exclude')
        with open(exclude_file,'a+') as f:
            for file in files:
                f.write(file + '\n')

    def push(self, home):
        git_cmd = 'git push -u onepanel master'
        p = subprocess.Popen(git_cmd, cwd=home, shell=True)
        p.wait()

    def pull(self, home):
        git_cmd = 'git pull'
        p = subprocess.Popen(git_cmd, cwd=home, shell=True)
        p.wait()



"""
Instance ("notebook") module
"""
import re

import click
import sys

from onepanel.commands.base import APIViewController
from onepanel.commands.login import login_required


class InstanceViewController(APIViewController):

    def __init__(self, conn):

        super(InstanceViewController, self).__init__(conn)

        project = self.get_project()

        self.endpoint = '{root}/accounts/{account_uid}/projects/{project_uid}/instances'.format(
            root=self.conn.URL,
            account_uid=project.account_uid,
            project_uid=project.project_uid
        )


@click.group(help='Notebook (instance) commands group')
@click.pass_context
def notebooks(ctx):
    ctx.obj['vc'] = InstanceViewController(ctx.obj['connection'])


@notebooks.command(
    'create',
    help='Create a new notebook. The notebook\'s name: Max 25 chars, lower case alphanumeric or "-", '
         'must start and end with alphanumeric'
)
@click.argument(
    'notebook_uid',
    type=str
)
@click.option(
    '-m', '--machine-type',
    type=str,
    required=True,
    help='Machine type ID. Call "onepanel machine_types list" for IDs.'
)
@click.option(
    '-e', '--environment',
    type=str,
    required=True,
    help='Instance template ID. Call "onepanel environments list" for IDs.'
)
@click.option(
    '-s', '--storage',
    type=str,
    required=True,
    help='Storage type ID.'
)
@click.pass_context
@login_required
def create_instance(ctx, notebook_uid, machine_type, environment, storage):
    instance_uid = notebook_uid
    machine_type_uid = machine_type
    instance_template_uid = environment
    volume_type_uid = storage

    pattern = re.compile('^[a-z0-9][-a-z0-9]{1,23}[a-z0-9]$')
    if not pattern.match(instance_uid):
        print(
            'Incorrect ID. ID must be:',
            '1. Maximum 25 chars',
            '2. Lower case alphanumeric or "-"',
            '3. Start and end with alphanumeric',
            sep='\n'
        )
        sys.exit(1)

    new_instance = {
        'uid': instance_uid,
        'machineType': {
            'uid': machine_type_uid
        },
        'volumeType': {
            'uid': volume_type_uid
        },
        'instanceTemplate': {
            'uid': instance_template_uid
        }
    }

    created_instance = ctx.obj['vc'].post(post_object=new_instance)

    if created_instance:
        print('New notebook ID: {}'.format(created_instance['uid']))


@notebooks.command('list', help='Show active notebooks in the current project')
@click.pass_context
@login_required
def list_instances(ctx):
    vc = ctx.obj['vc']
    items = vc.get()

    if items is None or len(items) == 0:
        print('No notebooks found.')
        return

    for item in items:
        info = item['machineType']['info']
        item['cpu'] = info['cpu']
        item['gpu'] = info.get('gpu')
        item['ram'] = info['ram']
        item['hdd'] = item['volumeType']['info']['size']
    vc.print_items(items, fields=['uid', 'cpu', 'gpu', 'ram', 'hdd'], field_names=['ID', 'CPU', 'GPU', 'RAM', 'HDD'])


@notebooks.command('terminate', help='Terminate the notebook')
@click.argument(
    'notebook_uid',
    type=str
)
@click.pass_context
@login_required
def terminate_instance(ctx, notebook_uid):
    ctx.obj['vc'].delete(notebook_uid, message_on_success='Notebook terminated')



""" Command line interface for the OnePanel Machine Learning platform

Entry point for command line interface.
"""

import os
import functools
import configparser
import click
import requests

from onepanel.commands.login import login, login_with_token
from onepanel.commands.projects import projects
from onepanel.commands.datasets import datasets
from onepanel.commands.jobs import jobs
from onepanel.commands.machine_types import machine_types
from onepanel.commands.environments import environments
from onepanel.commands.instances import notebooks
from onepanel.commands.volume_types import volume_types
from onepanel.commands.common import clone, push, pull

class Connection:
    """ REST API requests defaults and credentials
    """
    def __init__(self):
        self.URL = os.getenv('BASE_API_URL', 'https://c.onepanel.io/api')
        self.SSL_VERIFY = True
        self.headers = {'Content-Type': 'application/json'}
        self.account_uid = None
        self.token = None

        # wrap requests methods to reduce number of arguments in api queries
        self.get = functools.partial(requests.get, headers=self.headers, verify=self.SSL_VERIFY)
        self.post = functools.partial(requests.post, headers=self.headers, verify=self.SSL_VERIFY)
        self.put = functools.partial(requests.put, headers=self.headers, verify=self.SSL_VERIFY)
        self.delete = functools.partial(requests.delete, headers=self.headers, verify=self.SSL_VERIFY)

    def save_credentials(self, data):
        credentials = configparser.ConfigParser()
        credentials['default'] = {'uid': data['uid'],
                                  'token': data['sessions'][0]['token'],
                                  'account_uid': data['account']['uid']}

        onepanel_home = os.path.expanduser(os.path.join('~', '.onepanel'))
        if not os.path.exists(onepanel_home):
            os.makedirs(onepanel_home)

        filename = os.path.join(onepanel_home, 'credentials')
        with open(filename, 'w') as f:
            credentials.write(f)

    def load_credentials(self):
        credentials = configparser.ConfigParser()
        filename = os.path.expanduser(os.path.join('~', '.onepanel', 'credentials'))
        credentials.read(filename)

        self.account_uid = credentials.get('default', 'account_uid', fallback=None)
        self.token = credentials.get('default', 'token', fallback=None)

        if self.token:
            self.headers['Authorization'] = 'Bearer {}'.format(self.token)


@click.group()
@click.version_option(version='0.12.2', prog_name='Onepanel CLI')
@click.pass_context
def cli(ctx):
    conn = Connection()
    conn.load_credentials()

    ctx.obj = {}
    ctx.obj['connection'] = conn


cli.add_command(login)
cli.add_command(login_with_token)
cli.add_command(clone)
cli.add_command(push)
cli.add_command(pull)
cli.add_command(projects)
cli.add_command(datasets)
cli.add_command(jobs)
cli.add_command(machine_types)
cli.add_command(environments)
cli.add_command(volume_types)
cli.add_command(notebooks)


if __name__ == '__main__':
    cli()



""" Command line interface for the OnePanel Machine Learning platform

'Datasets' commands group.
"""

import os
import configobj
import json
import re
import click

import humanize
from prettytable import PrettyTable
from onepanel.commands.login import login_required
from onepanel.gitwrapper import GitWrapper


class Dataset:
    """ Datasets data model
    """

    DATASET_FILE = '.onepanel/dataset'
    EXCLUSIONS = ['.onepanel/dataset']

    def __init__(self, account_uid=None, dataset_uid=None):
        self.account_uid = account_uid
        self.dataset_uid = dataset_uid

    def save(self, home):
        if not os.path.exists(home):
            os.makedirs(home)
        onepanel_dir = os.path.join(home, '.onepanel')
        if not os.path.exists(onepanel_dir):
            os.makedirs(onepanel_dir)
        dataset_file = os.path.join(home, Dataset.DATASET_FILE)

        cfg = configobj.ConfigObj(dataset_file)
        cfg['uid'] = self.dataset_uid
        cfg['account_uid'] = self.account_uid
        cfg.write()

    @classmethod
    def from_json(cls, data):
        dataset = cls(data['account']['uid'], data['uid'])
        return dataset

    @classmethod
    def from_directory(cls, home):
        if not Dataset.exists_local(home):
            return None

        dataset_file = os.path.join(home, Dataset.DATASET_FILE)
        cfg = configobj.ConfigObj(dataset_file)
        dataset = cls(cfg['account_uid'], cfg['uid'])
        return dataset

    @staticmethod
    def is_uid_valid(uid):
        pattern = re.compile('^[a-z0-9][-a-z0-9]{1,23}[a-z0-9]$')
        if pattern.match(uid):
            return True
        else:
            return False

    @staticmethod
    def exists_local(home):
        dataset_file = os.path.join(home, Dataset.DATASET_FILE)
        if os.path.isfile(dataset_file):
            return True
        else:
            return False

    @staticmethod
    def exists_remote(dataset_uid, data):
        exists = False
        for item in data:
            if item['uid'] == dataset_uid:
                exists = True
                break
        return exists

    @staticmethod
    def print_list(data):
        if len(data) == 0:
            print('No datasets found')
            return

        tbl = PrettyTable(border=False)
        tbl.field_names = ['NAME', 'VERSIONS', 'SIZE']
        tbl.align = 'l'
        for row in data:
            tbl.add_row([row['uid'], row['statistics']['versionCount'], humanize.naturalsize(row['statistics']['size'], binary=True)])
        print(tbl)


def create_dataset(ctx, account_uid, home):
    """ Dataset creation method for 'datasets_init' and 'datasets_create'
    commands
    """

    conn = ctx.obj['connection']

    if not account_uid:
        account_uid = conn.account_uid

    dataset_uid = os.path.basename(home)
    if not Dataset.is_uid_valid(dataset_uid):
        click.echo('Dataset name {} is invalid.'.format(dataset_uid))
        click.echo('Name should be 3 to 25 characters long, lower case alphanumeric or \'-\'\n'
                   'must start and end with an alphanumeric character.')
        return None

    url = '{}/datasets'.format(conn.URL, account_uid)
    r = conn.get(url)
    if r.status_code == 200:
        remote_list = r.json()
    else:
        print('Error: {}'.format(r.status_code))
        return None

    dataset = None
    if Dataset.exists_remote(dataset_uid, remote_list):
        if Dataset.exists_local(home):
            click.echo('Dataset already initialized')
        else:
            dataset = Dataset(account_uid, dataset_uid)
            dataset.save(home)
            git = GitWrapper()
            git.init(home, account_uid, dataset_uid + '_dataset')
            git.exclude(home, Dataset.EXCLUSIONS)

    else:
        can_create = True
        if Dataset.exists_local(home):
            can_create = click.confirm(
                'Dataset does not exists for {}, create the dataset and overwrite?'
                .format(account_uid))

        if can_create:
            url = '{}/accounts/{}/datasets'.format(conn.URL, account_uid)
            data = {
                'uid': dataset_uid
            }
            r = conn.post(url, data=json.dumps(data))

            if r.status_code == 200:
                dataset = Dataset.from_json(r.json())
                dataset.save(home)
                git = GitWrapper()
                git.init(home, account_uid, dataset_uid + '_dataset')
                git.exclude(home, Dataset.EXCLUSIONS)
            else:
                print('Error: {}'.format(r.status_code))

    return dataset

@click.group(help='Dataset commands group')
@click.pass_context
def datasets(ctx):
    pass


@datasets.command('list', help='Display a list of all datasets.')
@click.pass_context
@login_required
def datasets_list(ctx):
    conn = ctx.obj['connection']
    url = '{}/datasets'.format(conn.URL)

    r = conn.get(url)
    if r.status_code == 200:
        Dataset.print_list(r.json())
    elif r.status_code == 404:
        print('No datasets found')
    else:
        print('Error: {}'.format(r.status_code))


@datasets.command('init', help='Initialize dataset in current directory.')
@click.option('--account',
              help='Dataset is created and initialized for this account_uid.')
@click.pass_context
@login_required
def datasets_init(ctx, account):
    home = os.getcwd()
    if not Dataset.is_uid_valid(os.path.basename(home)):
        dataset_uid = click.prompt('Please enter a valid dataset name')
        home = os.path.join(home, dataset_uid)

    if create_dataset(ctx, account, home):
        click.echo('Dataset is initialized in current directory.')


@datasets.command('create', help='Create dataset in new directory.')
@click.option('--name',
              prompt='Name', default=os.path.basename(os.getcwd()),
              help='Dataset uid.')
@click.option('--account',
              help='Dataset is created and initialized for this account_uid.')
@click.pass_context
@login_required
def datasets_create(ctx, name, account):
    home = os.path.join(os.getcwd(), name)
    if create_dataset(ctx, account, home):
        click.echo('Dataset is created in directory {}.'.format(home))


def datasets_clone(ctx, path, directory):
    conn = ctx.obj['connection']

    values = path.split('/')
    account_uid = conn.account_uid

    if len(values) == 3:
        try:
            account_uid, datasets_dir, dataset_uid = values
            assert (datasets_dir == 'datasets')
        except:
            click.echo('Invalid dataset path. Please use <account_uid>/datasets/<uid>')
            return
    else:
        click.echo('Invalid dataset path. Please use <account_uid>/datasets/<uid>')
        return

    # check dataset path, account_uid, dataset_uid
    if directory is None:
        home = os.path.join(os.getcwd(), dataset_uid)
    elif directory == '.':
        home = os.getcwd()
    else:
        home = os.path.join(os.getcwd(), directory)

    # check if the dataset exisits
    r = conn.get('{}/datasets'.format(conn.URL, account_uid))
    if r.status_code == 200:
        remote_list = r.json()
    else:
        print('Error: {}'.format(r.status_code))
        return

    if not Dataset.exists_remote(dataset_uid, remote_list):
        click.echo('There is no dataset {}/{} on the server'.format(account_uid, dataset_uid))
        return

    can_create = True
    if Dataset.exists_local(home):
        can_create = click.confirm('Dataset already exists, overwrite?')
    if not can_create:
        return

    # git clone
    git = GitWrapper()
    if git.clone(home, account_uid, dataset_uid, '_dataset') == 0:
        Dataset(account_uid, dataset_uid).save(home)
        git.exclude(home, Dataset.EXCLUSIONS)





from setuptools import setup

setup(
    name="onepanel",
    version='0.12.3',
    packages = ['onepanel', 'onepanel.commands'],
    install_requires=[
        'requests',
        'click',
        'PTable',
        'configobj',
        'websocket',
        'humanize'
    ],
    entry_points='''
        [console_scripts]
        onepanel=onepanel.cli:cli
    ''',
)



""" Command line interface for the OnePanel Machine Learning platform

'Projects' commands group.
"""

import os
import configobj
import json
import re
import click
from prettytable import PrettyTable
from onepanel.commands.login import login_required
from onepanel.gitwrapper import GitWrapper


class Project:
    """ Projects data model
    """

    PROJECT_FILE = '.onepanel/project'
    EXCLUSIONS = ['.onepanel/project']

    def __init__(self, account_uid=None, project_uid=None):
        self.account_uid = account_uid
        self.project_uid = project_uid

    def save(self, home):
        if not os.path.exists(home):
            os.makedirs(home)
        onepanel_dir = os.path.join(home, '.onepanel')
        if not os.path.exists(onepanel_dir):
            os.makedirs(onepanel_dir)
        project_file = os.path.join(home, Project.PROJECT_FILE)

        cfg = configobj.ConfigObj(project_file)
        cfg['uid'] = self.project_uid
        cfg['account_uid'] = self.account_uid
        cfg.write()

    @classmethod
    def from_json(cls, data):
        project = cls(data['account']['uid'], data['uid'])
        return project

    @classmethod
    def from_directory(cls, home):
        if not Project.exists_local(home):
            return None

        project_file = os.path.join(home, Project.PROJECT_FILE)
        cfg = configobj.ConfigObj(project_file)
        project = cls(cfg['account_uid'], cfg['uid'])
        return project

    @staticmethod
    def is_uid_valid(uid):
        pattern = re.compile('^[a-z0-9][-a-z0-9]{1,23}[a-z0-9]$')
        if pattern.match(uid):
            return True
        else:
            return False

    @staticmethod
    def exists_local(home):
        project_file = os.path.join(home, Project.PROJECT_FILE)
        if os.path.isfile(project_file):
            return True
        else:
            return False

    @staticmethod
    def exists_remote(project_uid, data):
        exists = False
        for item in data:
            if item['uid'] == project_uid:
                exists = True
                break
        return exists

    @staticmethod
    def print_list(data):
        if len(data) == 0:
            print('No projects found')
            return

        tbl = PrettyTable(border=False)
        tbl.field_names = ['NAME', 'INSTANCES', 'JOBS']
        tbl.align = 'l'
        for row in data:
            tbl.add_row([row['uid'], row['instanceCount'], row['jobCount']])
        print(tbl)


def create_project(ctx, account_uid, home):
    """ Project creation method for 'projects_init' and 'projects_create'
    commands
    """

    conn = ctx.obj['connection']

    if not account_uid:
        account_uid = conn.account_uid

    project_uid = os.path.basename(home)
    if not Project.is_uid_valid(project_uid):
        click.echo('Project name {} is invalid.'.format(project_uid))
        click.echo('Name should be 3 to 25 characters long, lower case alphanumeric or \'-\'\n'
                   'must start and end with an alphanumeric character.')
        return None

    url = '{}/projects'.format(conn.URL, account_uid)
    r = conn.get(url)
    if r.status_code == 200:
        remote_list = r.json()
    else:
        print('Error: {}'.format(r.status_code))
        return None

    project = None
    if Project.exists_remote(project_uid, remote_list):
        if Project.exists_local(home):
            click.echo('Project already initialized')
        else:
            project = Project(account_uid, project_uid)
            project.save(home)
            git = GitWrapper()
            git.init(home, account_uid, project_uid)
            git.exclude(home, Project.EXCLUSIONS)

    else:
        can_create = True
        if Project.exists_local(home):
            can_create = click.confirm(
                'Project does not exists for {}, create the project and overwrite?'
                .format(account_uid))

        if can_create:
            url = '{}/accounts/{}/projects'.format(conn.URL, account_uid)
            data = {
                'uid': project_uid
            }
            r = conn.post(url, data=json.dumps(data))

            if r.status_code == 200:
                project = Project.from_json(r.json())
                project.save(home)
                git = GitWrapper()
                git.init(home, account_uid, project_uid)
                git.exclude(home, Project.EXCLUSIONS)
            else:
                print('Error: {}'.format(r.status_code))

    return project

@click.group(help='Project commands group')
@click.pass_context
def projects(ctx):
    pass


@projects.command('list', help='Display a list of all projects.')
@click.pass_context
@login_required
def projects_list(ctx):
    conn = ctx.obj['connection']
    url = '{}/projects'.format(conn.URL)

    r = conn.get(url)
    if r.status_code == 200:
        Project.print_list(r.json())
    elif r.status_code == 404:
        print('No projects found')
    else:
        print('Error: {}'.format(r.status_code))


@projects.command('init', help='Initialize project in current directory.')
@click.option('--account',
              help='Project is created and initialized for this account_uid.')
@click.pass_context
@login_required
def projects_init(ctx, account):
    home = os.getcwd()
    if not Project.is_uid_valid(os.path.basename(home)):
        project_uid = click.prompt('Please enter a valid project name')
        home = os.path.join(home, project_uid)

    if create_project(ctx, account, home):
        click.echo('Project is initialized in current directory.')


@projects.command('create', help='Create project in new directory.')
@click.option('--name',
              prompt='Name', default=os.path.basename(os.getcwd()),
              help='Project uid.')
@click.option('--account',
              help='Project is created and initialized for this account_uid.')
@click.pass_context
@login_required
def projects_create(ctx, name, account):
    home = os.path.join(os.getcwd(), name)
    if create_project(ctx, account, home):
        click.echo('Project is created in directory {}.'.format(home))


def projects_clone(ctx, path, directory):
    conn = ctx.obj['connection']

    values = path.split('/')
    account_uid = conn.account_uid

    if len(values) == 3:
        try:
            account_uid, projects_dir, project_uid = values
            assert (projects_dir == 'projects')
        except:
            click.echo('Invalid project path. Please use <account_uid>/projects/<uid>')
            return
    else:
        click.echo('Invalid project path. Please use <account_uid>/projects/<uid>')
        return

    # check project path, account_uid, project_uid
    if directory is None:
        home = os.path.join(os.getcwd(), project_uid)
    elif directory == '.':
        home = os.getcwd()
    else:
        home = os.path.join(os.getcwd(), directory)

    # check if the project exisits
    r = conn.get('{}/projects'.format(conn.URL, account_uid))
    if r.status_code == 200:
        remote_list = r.json()
    else:
        print('Error: {}'.format(r.status_code))
        return

    if not Project.exists_remote(project_uid, remote_list):
        click.echo('There is no project {}/{} on the server'.format(account_uid, project_uid))
        return

    can_create = True
    if Project.exists_local(home):
        can_create = click.confirm('Project already exists, overwrite?')
    if not can_create:
        return

    # git clone
    git = GitWrapper()
    if git.clone(home, account_uid, project_uid) == 0:
        Project(account_uid, project_uid).save(home)
        git.exclude(home, Project.EXCLUSIONS)





import os
import click

from onepanel.commands.login import login_required
from onepanel.commands.projects import projects_clone
from onepanel.commands.datasets import datasets_clone
from onepanel.gitwrapper import GitWrapper

def get_entity_type(path):
    dirs = path.split('/')
    entity_type = None
    if len(dirs) == 3:
        account_uid, dir, uid = dirs
        if dir == 'projects':
            entity_type = 'project'
        elif dir == 'datasets':
            entity_type = 'dataset'
    return entity_type

@click.command('clone', help='Clone <account_uid>/<uid> project from server.')
@click.argument('path', type=click.Path())
@click.argument('directory', type=click.Path(), required=False)
@click.pass_context
@login_required
def clone(ctx, path, directory):
    entity_type = get_entity_type(path)
    if entity_type == 'project':
        projects_clone(ctx, path, directory)
    elif entity_type == 'dataset':
        datasets_clone(ctx, path, directory)
    else:
        click.echo('Invalid project or dataset path.')


@click.command('push', help='Push changes to onepanel')
@click.pass_context
@login_required
def push(ctx):
    home = os.getcwd()
    GitWrapper().push(home)
        
@click.command('pull', help='Pull changes from onepanel (fetch and merge)')
@click.pass_context
@login_required
def pull(ctx):
    home = os.getcwd()
    GitWrapper().pull(home)
